@@ -9,8 +9,6 @@
 +----------------------------------------------------------------------+
 */
 
-/* $Id$ */
-
 #include "php_oauth.h"
 #include "provider.h"
 
@@ -35,9 +33,15 @@ static zend_object* php_so_object_new(zend_class_entry *ce) /* {{{ */
 }
 /* }}} */
 
+#if PHP_VERSION_ID < 80000
 static zend_object *oauth_clone_obj(zval *this_ptr) /* {{{ */
 {
 	php_so_object *old_obj = Z_SOO_P(this_ptr);
+#else
+static zend_object *oauth_clone_obj(zend_object *this_ptr) /* {{{ */
+{
+	php_so_object *old_obj = so_object_from_obj(this_ptr);
+#endif
 	php_so_object *new_obj = so_object_from_obj(php_so_object_new(old_obj->zo.ce));
 
 	zend_objects_clone_members(&new_obj->zo, &old_obj->zo);
@@ -178,25 +182,24 @@ void soo_handle_error(php_so_object *soo, long errorCode, char *msg, char *respo
 	if (!errorCode) {
 		php_error(E_WARNING, "caller did not pass an errorcode!");
 	} else {
-		zend_update_property_long(dex, &ex, "code", sizeof("code")-1, errorCode);
+		zend_update_property_long(dex, OBJ_FOR_PROP(&ex), "code", sizeof("code")-1, errorCode);
 	}
 	if (response) {
-		zend_update_property_string(dex, &ex, "lastResponse", sizeof("lastResponse")-1, response);
+		zend_update_property_string(dex, OBJ_FOR_PROP(&ex), "lastResponse", sizeof("lastResponse")-1, response);
 	}
 	if(soo && soo->debug && Z_TYPE(soo->debugArr) != IS_UNDEF) {
-		zend_update_property(dex, &ex, "debugInfo", sizeof("debugInfo") - 1, &soo->debugArr);
+		zend_update_property(dex, OBJ_FOR_PROP(&ex), "debugInfo", sizeof("debugInfo") - 1, &soo->debugArr);
 	}
 
 	if(additional_info) {
-		zend_update_property_string(dex, &ex, "additionalInfo", sizeof("additionalInfo")-1, additional_info);
+		zend_update_property_string(dex, OBJ_FOR_PROP(&ex), "additionalInfo", sizeof("additionalInfo")-1, additional_info);
 	}
 
-	zend_update_property_string(dex, &ex, "message", sizeof("message")-1, msg);
+	zend_update_property_string(dex, OBJ_FOR_PROP(&ex), "message", sizeof("message")-1, msg);
 	zend_throw_exception_object(&ex);
 }
 /* }}} */
 
-
 zend_string *soo_sign_hmac(php_so_object *soo, char *message, const char *cs, const char *ts, const oauth_sig_context *ctx) /* {{{ */
 {
 	zval args[4], retval, func;
@@ -234,7 +237,7 @@ zend_string *soo_sign_hmac(php_so_object *soo, char *message, const char *cs, co
 }
 /* }}} */
 
-zend_string *soo_sign_rsa(php_so_object *soo, char *message, const oauth_sig_context *ctx)
+zend_string *soo_sign_rsa(php_so_object *soo, char *message, const oauth_sig_context *ctx) /* {{{ */
 {
 	zval args[3], func, retval;
 	zend_string *result;
@@ -249,12 +252,16 @@ zend_string *soo_sign_rsa(php_so_object *soo, char *message, const oauth_sig_con
 	/* TODO: add support for other algorithms instead of OPENSSL_ALGO_SHA1 */
 
 	ZVAL_STRING(&args[0], message);
+	ZVAL_NULL(&args[1]);
+	ZVAL_MAKE_REF(&args[1]);
 	ZVAL_DUP(&args[2], &ctx->privatekey);
 
-	call_user_function_ex(EG(function_table), NULL, &func, &retval, 3, args, 0, NULL);
+	call_user_function(EG(function_table), NULL, &func, &retval, 3, args);
 
 	if (Z_TYPE(retval) == IS_TRUE || Z_TYPE(retval) == IS_FALSE) {
-		result = php_base64_encode((unsigned char *) Z_STRVAL_P(Z_REFVAL(args[1])), Z_STRLEN_P(Z_REFVAL(args[1])));
+		zend_string *sig_str = zval_get_string(&args[1]);
+		result = php_base64_encode((unsigned char *) sig_str->val, sig_str->len);
+		zend_string_release(sig_str);
 		zval_ptr_dtor(&args[1]);
 	} else {
 		result = NULL;
@@ -274,7 +281,7 @@ zend_string *soo_sign_plain(php_so_object *soo, const char *cs, const char *ts)
 }
 /* }}} */
 
-oauth_sig_context *oauth_create_sig_context(const char *sigmethod)
+oauth_sig_context *oauth_create_sig_context(const char *sigmethod) /* {{{ */
 {
 	oauth_sig_context *ctx;
 
@@ -291,8 +298,9 @@ oauth_sig_context *oauth_create_sig_context(const char *sigmethod)
 
 	return ctx;
 }
+/* }}} */
 
-zend_string *soo_sign(php_so_object *soo, char *message, zval *cs, zval *ts, const oauth_sig_context *ctx)
+zend_string *soo_sign(php_so_object *soo, char *message, zval *cs, zval *ts, const oauth_sig_context *ctx) /* {{{ */
 {
 	const char *csec = cs?Z_STRVAL_P(cs):"", *tsec = ts?Z_STRVAL_P(ts):"";
 
@@ -305,6 +313,7 @@ zend_string *soo_sign(php_so_object *soo, char *message, zval *cs, zval *ts, con
 	}
 	return NULL;
 }
+/* }}} */
 
 static inline zval *soo_get_property(php_so_object *soo, char *prop_name) /* {{{ */
 {
@@ -340,7 +349,7 @@ zend_string *oauth_url_encode(char *url, int url_len) /* {{{ */
 }
 /* }}} */
 
-zend_string *oauth_http_encode_value(zval *v)
+zend_string *oauth_http_encode_value(zval *v) /* {{{ */
 {
 	zend_string *param_value = NULL;
 
@@ -356,8 +365,9 @@ zend_string *oauth_http_encode_value(zval *v)
 
 	return param_value;
 }
+/* }}} */
 
-static int oauth_strcmp(zval *first, zval *second)
+static int oauth_strcmp(zval *first, zval *second) /* {{{ */
 {
 	int result;
 	result = string_compare_function(first, second);
@@ -370,23 +380,33 @@ static int oauth_strcmp(zval *first, zval *second)
 
 	return 0;
 }
+/* }}} */
 
-static int oauth_compare_value(const void *a, const void *b)
+#if PHP_VERSION_ID < 80000
+static int oauth_compare_value(const void *a, const void *b) /* {{{ */
 {
 	Bucket *f, *s;
 	f = (Bucket *)a;
 	s = (Bucket *)b;
 
+#else
+static int oauth_compare_value(Bucket *f, Bucket *s) /* {{{ */
+{
+#endif
 	return oauth_strcmp(&f->val, &s->val);
 }
+/* }}} */
 
-static int oauth_compare_key(const void *a, const void *b)
+#if PHP_VERSION_ID < 80000
+static int oauth_compare_key(const void *a, const void *b) /* {{{ */
 {
+    Bucket *f = (Bucket *)a, *s = (Bucket *)b;
+#else
+static int oauth_compare_key(Bucket *f, Bucket *s) /* {{{ */
+{
+#endif
 	zval first, second;
 	int result;
-    Bucket *f, *s;
-	f = (Bucket *) a;
-	s = (Bucket *) b;
 
 	if (f->key == NULL) {
 		ZVAL_LONG(&first, f->h);
@@ -405,14 +425,15 @@ static int oauth_compare_key(const void *a, const void *b)
 	zval_ptr_dtor(&second);
 	return result;
 }
+/* }}} */
 
 /* build url-encoded string from args, optionally starting with & */
-int oauth_http_build_query(php_so_object *soo, smart_string *s, HashTable *args, zend_bool prepend_amp)
+int oauth_http_build_query(php_so_object *soo, smart_string *s, HashTable *args, zend_bool prepend_amp) /* {{{ */
 {
 	zval *cur_val;
 	zend_string *cur_key, *arg_key, *param_value;
 	int numargs = 0, hash_key_type, skip_append = 0, i, found;
-	ulong num_index;
+	zend_ulong num_index;
 	HashPosition pos;
 	smart_string keyname;
 
@@ -537,9 +558,10 @@ int oauth_http_build_query(php_so_object *soo, smart_string *s, HashTable *args,
 	}
 	return numargs;
 }
+/* }}} */
 
 /* retrieves parameter value from the _GET or _POST superglobal */
-void get_request_param(char *arg_name, char **return_val, int *return_len)
+void get_request_param(char *arg_name, char **return_val, int *return_len) /* {{{ */
 {
 	zval *ptr;
 	if (
@@ -553,12 +575,12 @@ void get_request_param(char *arg_name, char **return_val, int *return_len)
 	*return_val = NULL;
 	*return_len = 0;
 }
+/* }}} */
 
 /*
  * This function does not currently care to respect parameter precedence, in the sense that if a common param is defined
  * in POST/GET or Authorization header, the precendence is defined by: OAuth Core 1.0 section 9.1.1
  */
-
 zend_string *oauth_generate_sig_base(php_so_object *soo, const char *http_method, const char *uri, HashTable *post_args, HashTable *extra_args) /* {{{ */
 {
 	zval params;
@@ -665,7 +687,8 @@ zend_string *oauth_generate_sig_base(php_so_object *soo, const char *http_method
 }
 /* }}} */
 
-static void oauth_set_debug_info(php_so_object *soo) {
+static void oauth_set_debug_info(php_so_object *soo) /* {{{ */
+{
 	zval *debugInfo;
 	if (soo->debug_info) {
 		debugInfo = &soo->debugArr;
@@ -685,11 +708,12 @@ static void oauth_set_debug_info(php_so_object *soo) {
 		ADD_DEBUG_INFO(debugInfo, "body_recv", soo->debug_info->body_in, 0);
 		ADD_DEBUG_INFO(debugInfo, "info", soo->debug_info->curl_info, 0);
 
-		zend_update_property(soo_class_entry, soo->this_ptr, "debugInfo", sizeof("debugInfo") - 1, debugInfo);
+		zend_update_property(soo_class_entry, OBJ_FOR_PROP(soo->this_ptr), "debugInfo", sizeof("debugInfo") - 1, debugInfo);
 	} else {
 		ZVAL_UNDEF(&soo->debugArr);
 	}
 }
+/* }}} */
 
 static int add_arg_for_req(HashTable *ht, const char *arg, const char *val) /* {{{ */
 {
@@ -702,7 +726,7 @@ static int add_arg_for_req(HashTable *ht, const char *arg, const char *val) /* {
 }
 /* }}} */
 
-void oauth_add_signature_header(HashTable *request_headers, HashTable *oauth_args, smart_string *header)
+void oauth_add_signature_header(HashTable *request_headers, HashTable *oauth_args, smart_string *header) /* {{{ */
 {
 	smart_string sheader = {0};
 	zend_bool prepend_comma = FALSE;
@@ -710,14 +734,15 @@ void oauth_add_signature_header(HashTable *request_headers, HashTable *oauth_arg
 	zval *curval;
 	zend_string *param_name, *param_val;
 	zend_string *cur_key;
-	ulong num_key;
+	zend_ulong num_key;
+	HashPosition pos;
 
 	smart_string_appends(&sheader, "OAuth ");
 
-	for (zend_hash_internal_pointer_reset(oauth_args);
-			(curval = zend_hash_get_current_data(oauth_args)) != NULL;
-			zend_hash_move_forward(oauth_args)) {
-		zend_hash_get_current_key(oauth_args, &cur_key, &num_key);
+	for (zend_hash_internal_pointer_reset_ex(oauth_args, &pos);
+			(curval = zend_hash_get_current_data_ex(oauth_args, &pos)) != NULL;
+				zend_hash_move_forward_ex(oauth_args, &pos)) {
+		zend_hash_get_current_key_ex(oauth_args, &cur_key, &num_key, &pos);
 
 		if (prepend_comma) {
 			smart_string_appendc(&sheader, ',');
@@ -744,6 +769,7 @@ void oauth_add_signature_header(HashTable *request_headers, HashTable *oauth_arg
 	}
 	smart_string_free(&sheader);
 }
+/* }}} */
 
 #define HTTP_RESPONSE_CAAS(zvalp, header, storkey) { \
 	if (0==strncasecmp(Z_STRVAL_P(zvalp),header,sizeof(header)-1)) { \
@@ -782,8 +808,8 @@ static long make_req_streams(php_so_object *soo, const char *url, const smart_st
 	sc = php_stream_context_alloc();
 
 	if (payload->len) {
-		smart_string_0(payload);
 		ZVAL_STRINGL(&zpayload, payload->c, payload->len);
+		Z_STRVAL(zpayload)[Z_STRLEN(zpayload)] = '\0';
 		php_stream_context_set_option(sc, "http", "content", &zpayload);
 		zval_ptr_dtor(&zpayload);
 		/**
@@ -797,7 +823,7 @@ static long make_req_streams(php_so_object *soo, const char *url, const smart_st
 		HashPosition pos;
 		zval *cur_val, zheaders;
 		zend_string *cur_key;
-		ulong num_key;
+		zend_ulong num_key;
 		smart_string sheaders = {0};
 		int first = 1;
 
@@ -806,7 +832,7 @@ static long make_req_streams(php_so_object *soo, const char *url, const smart_st
 				zend_hash_move_forward_ex(request_headers, &pos)) {
 			/* check if a string based key is used */
 			smart_string sheaderline = {0};
-			switch (zend_hash_get_current_key_ex(request_headers, &cur_key, &num_key, &pos)) {
+			switch ((int)zend_hash_get_current_key_ex(request_headers, &cur_key, &num_key, &pos)) {
 				case HASH_KEY_IS_STRING:
 					smart_string_appendl(&sheaderline, ZSTR_VAL(cur_key), ZSTR_LEN(cur_key));
 					break;
@@ -936,7 +962,7 @@ static long make_req_streams(php_so_object *soo, const char *url, const smart_st
 #if OAUTH_USE_CURL
 static size_t soo_read_response(char *ptr, size_t size, size_t nmemb, void *ctx) /* {{{ */
 {
-	uint relsize;
+	size_t relsize;
 	php_so_object *soo = (php_so_object *)ctx;
 
 	relsize = size * nmemb;
@@ -988,7 +1014,7 @@ int oauth_debug_handler(CURL *ch, curl_infotype type, char *data, size_t data_le
 }
 /* }}} */
 
-static size_t soo_read_header(void *ptr, size_t size, size_t nmemb, void *ctx)
+static size_t soo_read_header(void *ptr, size_t size, size_t nmemb, void *ctx) /* {{{ */
 {
 	char *header;
 	size_t hlen, vpos = sizeof("Location:") - 1;
@@ -1024,6 +1050,7 @@ static size_t soo_read_header(void *ptr, size_t size, size_t nmemb, void *ctx)
 	}
 	return hlen;
 }
+/* }}} */
 
 long make_req_curl(php_so_object *soo, const char *url, const smart_string *payload, const char *http_method, HashTable *request_headers) /* {{{ */
 {
@@ -1034,10 +1061,11 @@ long make_req_curl(php_so_object *soo, const char *url, const smart_string *payl
 	double d_code;
 	zval info, *zca_info, *zca_path, *cur_val;
 	char *s_code, *content_type = NULL, *bufz = NULL;
-	uint sslcheck;
-	ulong num_key;
+	uint32_t sslcheck;
+	zend_ulong num_key;
 	smart_string sheader = {0};
 	zend_string *cur_key;
+	HashPosition pos;
 
 	zca_info = soo_get_property(soo, OAUTH_ATTR_CA_INFO);
 	zca_path = soo_get_property(soo, OAUTH_ATTR_CA_PATH);
@@ -1046,11 +1074,11 @@ long make_req_curl(php_so_object *soo, const char *url, const smart_string *payl
 	curl = curl_easy_init();
 
 	if (request_headers) {
-		for (zend_hash_internal_pointer_reset(request_headers);
-				(cur_val = zend_hash_get_current_data(request_headers)) != NULL;
-				zend_hash_move_forward(request_headers)) {
+		for (zend_hash_internal_pointer_reset_ex(request_headers, &pos);
+				(cur_val = zend_hash_get_current_data_ex(request_headers, &pos)) != NULL;
+				zend_hash_move_forward_ex(request_headers, &pos)) {
 			/* check if a string based key is used */
-			switch (zend_hash_get_current_key(request_headers, &cur_key, &num_key)) {
+			switch ((int)zend_hash_get_current_key_ex(request_headers, &cur_key, &num_key, &pos)) {
 				case HASH_KEY_IS_STRING:
 					smart_string_appendl(&sheader, ZSTR_VAL(cur_key), ZSTR_LEN(cur_key));
 					break;
@@ -1398,10 +1426,10 @@ static long oauth_fetch(php_so_object *soo, const char *url, const char *method,
 	zval *ts = NULL, *token_secret = NULL;
 	zval zret;
 	HashTable *oauth_args = NULL;
-	HashTable *rargs = NULL, *rheaders = NULL;
+	HashTable *rargs = NULL, rheaders;
 	long http_response_code, auth_type;
 	smart_string surl = {0}, payload = {0}, postdata = {0};
-	uint is_redirect = FALSE, follow_redirects = 0, need_to_free_rheaders = 0;
+	uint32_t is_redirect = FALSE, follow_redirects = 0;
 
 	auth_type = Z_LVAL_P(soo_get_property(soo, OAUTH_ATTR_AUTHMETHOD));
 	if(fetch_flags & OAUTH_OVERRIDE_HTTP_METHOD) {
@@ -1440,12 +1468,9 @@ static long oauth_fetch(php_so_object *soo, const char *url, const char *method,
 	}
 
 	/* additional http headers can be passed */
-	if (!request_headers) {
-		ALLOC_HASHTABLE(rheaders);
-		zend_hash_init(rheaders, 0, NULL, ZVAL_PTR_DTOR, 0);
-		need_to_free_rheaders = 1;
-	} else {
-		rheaders = HASH_OF(request_headers);
+	zend_hash_init(&rheaders, 0, NULL, ZVAL_PTR_DTOR, 0);
+	if (request_headers && zend_hash_num_elements(Z_ARRVAL_P(request_headers))) {
+		zend_hash_copy(&rheaders, Z_ARRVAL_P(request_headers), (copy_ctor_func_t) zval_add_ref);
 	}
 
 	/* initialize base url */
@@ -1508,9 +1533,7 @@ static long oauth_fetch(php_so_object *soo, const char *url, const char *method,
 			FREE_ARGS_HASH(oauth_args);
 			smart_string_free(&surl);
 			smart_string_free(&postdata);
-			if(need_to_free_rheaders) {
-				FREE_ARGS_HASH(rheaders);
-			}
+			zend_hash_destroy(&rheaders);
 			return SUCCESS;
 		}
 
@@ -1525,14 +1548,12 @@ static long oauth_fetch(php_so_object *soo, const char *url, const char *method,
 
 		if(fetch_flags & OAUTH_FETCH_HEADONLY) {
 			INIT_smart_string(soo->headers_out);
-			oauth_add_signature_header(rheaders, oauth_args, &soo->headers_out);
+			oauth_add_signature_header(&rheaders, oauth_args, &soo->headers_out);
 			smart_string_0(&payload);
 			FREE_ARGS_HASH(oauth_args);
 			smart_string_free(&surl);
 			smart_string_free(&postdata);
-			if(need_to_free_rheaders) {
-				FREE_ARGS_HASH(rheaders);
-			}
+			zend_hash_destroy(&rheaders);
 			return SUCCESS;
 		}
 
@@ -1561,7 +1582,7 @@ static long oauth_fetch(php_so_object *soo, const char *url, const char *method,
 				break;
 			case OAUTH_AUTH_TYPE_AUTHORIZATION:
 				/* add http header with oauth parameters */
-				oauth_add_signature_header(rheaders, oauth_args, NULL);
+				oauth_add_signature_header(&rheaders, oauth_args, NULL);
 				break;
 		}
 
@@ -1577,11 +1598,11 @@ static long oauth_fetch(php_so_object *soo, const char *url, const char *method,
 
 		switch (soo->reqengine) {
 			case OAUTH_REQENGINE_STREAMS:
-				http_response_code = make_req_streams(soo, surl.c, &payload, final_http_method, rheaders);
+				http_response_code = make_req_streams(soo, surl.c, &payload, final_http_method, &rheaders);
 				break;
 #if OAUTH_USE_CURL
 			case OAUTH_REQENGINE_CURL:
-				http_response_code = make_req_curl(soo, surl.c, &payload, final_http_method, rheaders);
+				http_response_code = make_req_curl(soo, surl.c, &payload, final_http_method, &rheaders);
 				if (soo->multipart_files_num) {
 					efree(soo->multipart_files);
 					efree(soo->multipart_params);
@@ -1647,14 +1668,14 @@ static long oauth_fetch(php_so_object *soo, const char *url, const char *method,
 
 	smart_string_free(&surl);
 	smart_string_free(&postdata);
-	if(need_to_free_rheaders) {
-		FREE_ARGS_HASH(rheaders);
-	}
+	zend_hash_destroy(&rheaders);
 
 	return http_response_code;
 }
 /* }}} */
 
+/* {{{ proto bool setRSACertificate(string $cert)
+       Sets the RSA certificate */
 SO_METHOD(setRSACertificate)
 {
 	char *key;
@@ -1679,15 +1700,19 @@ SO_METHOD(setRSACertificate)
 	zval_ptr_dtor(&args[0]);
 	zval_ptr_dtor(&func);
 
-	if (Z_TYPE(retval) == IS_RESOURCE) {
+	switch (Z_TYPE(retval)) {
+	case IS_RESOURCE:
+	case IS_OBJECT:
 		OAUTH_SIGCTX_SET_PRIVATEKEY(soo->sig_ctx, retval);
 		RETURN_TRUE;
-	} else {
+		break;
+	default:
 		zval_ptr_dtor(&retval);
 		soo_handle_error(soo, OAUTH_ERR_INTERNAL_ERROR, "Could not parse RSA certificate", NULL, NULL);
 		return;
 	}
 }
+/* }}} */
 
 /* {{{ proto string oauth_urlencode(string uri)
    URI encoding according to RFC 3986, note: is not utf8 capable until the underlying phpapi is */
@@ -1792,11 +1817,13 @@ SO_METHOD(__construct)
 	INIT_smart_string(soo->headers_in);
 
 	/* set default class members */
-	zend_update_property_null(soo_class_entry, obj, "debugInfo", sizeof("debugInfo") - 1);
-	zend_update_property_bool(soo_class_entry, obj, "debug", sizeof("debug") - 1, soo->debug);
-	zend_update_property_long(soo_class_entry, obj, "sslChecks", sizeof("sslChecks") - 1, soo->sslcheck);
+	zend_update_property_null(soo_class_entry, OBJ_FOR_PROP(obj), "debugInfo", sizeof("debugInfo") - 1);
+	zend_update_property_bool(soo_class_entry, OBJ_FOR_PROP(obj), "debug", sizeof("debug") - 1, soo->debug);
+	zend_update_property_long(soo_class_entry, OBJ_FOR_PROP(obj), "sslChecks", sizeof("sslChecks") - 1, soo->sslcheck);
 
+#if PHP_VERSION_ID < 80000
 	TSRMLS_SET_CTX(soo->thread_ctx);
+#endif
 
 	if (!sig_method_len) {
 		sig_method = OAUTH_SIG_METHOD_HMACSHA1;
@@ -1859,7 +1886,7 @@ SO_METHOD(__construct)
 }
 /* }}} */
 
-void oauth_free_privatekey(zval *privatekey)
+void oauth_free_privatekey(zval *privatekey) /* {{{ */
 {
 	zval func, retval;
 	zval args[1];
@@ -1876,6 +1903,7 @@ void oauth_free_privatekey(zval *privatekey)
 
 	zval_ptr_dtor(privatekey);
 }
+/* }}} */
 
 /* {{{ proto array OAuth::setCAPath(string ca_path, string ca_info)
    Set the Certificate Authority information */
@@ -2040,7 +2068,7 @@ SO_METHOD(disableDebug)
 	}
 
 	soo->debug = 0;
-	zend_update_property_bool(soo_class_entry, obj, "debug", sizeof("debug") - 1, 0);
+	zend_update_property_bool(soo_class_entry, OBJ_FOR_PROP(obj), "debug", sizeof("debug") - 1, 0);
 
 	RETURN_TRUE;
 }
@@ -2061,7 +2089,7 @@ SO_METHOD(enableDebug)
 	}
 
 	soo->debug = 1;
-	zend_update_property_bool(soo_class_entry, obj, "debug", sizeof("debug") - 1, 1);
+	zend_update_property_bool(soo_class_entry, OBJ_FOR_PROP(obj), "debug", sizeof("debug") - 1, 1);
 
 	RETURN_TRUE;
 }
@@ -2082,7 +2110,7 @@ SO_METHOD(enableSSLChecks)
 	}
 
 	soo->sslcheck = OAUTH_SSLCHECK_BOTH;
-	zend_update_property_long(soo_class_entry, obj, "sslChecks", sizeof("sslChecks") - 1, 1);
+	zend_update_property_long(soo_class_entry, OBJ_FOR_PROP(obj), "sslChecks", sizeof("sslChecks") - 1, 1);
 
 	RETURN_TRUE;
 }
@@ -2103,13 +2131,12 @@ SO_METHOD(disableSSLChecks)
 	}
 
 	soo->sslcheck = OAUTH_SSLCHECK_NONE;
-	zend_update_property_long(soo_class_entry, obj, "sslChecks", sizeof("sslChecks") - 1, 0);
+	zend_update_property_long(soo_class_entry, OBJ_FOR_PROP(obj), "sslChecks", sizeof("sslChecks") - 1, 0);
 
 	RETURN_TRUE;
 }
 /* }}} */
 
-
 /* {{{ proto bool OAuth::setSSLChecks(long sslcheck)
    Tweak specific SSL checks for requests (be careful using this for production) */
 SO_METHOD(setSSLChecks)
@@ -2127,14 +2154,13 @@ SO_METHOD(setSSLChecks)
 
 	soo->sslcheck = sslcheck & OAUTH_SSLCHECK_BOTH;
 
-	zend_update_property_long(soo_class_entry, obj, "sslChecks", sizeof("sslChecks") - 1,
+	zend_update_property_long(soo_class_entry, OBJ_FOR_PROP(obj), "sslChecks", sizeof("sslChecks") - 1,
 			soo->sslcheck);
 
 	RETURN_TRUE;
 }
 /* }}} */
 
-
 /* {{{ proto bool OAuth::setVersion(string version)
    Set oauth_version for requests (default 1.0) */
 SO_METHOD(setVersion)
@@ -2248,6 +2274,8 @@ SO_METHOD(setNonce)
 }
 /* }}} */
 
+/* {{{ proto bool setTimestamp(string $timestamp)
+Sets the OAuth timestamp for subsequent requests */
 SO_METHOD(setTimestamp)
 {
 	php_so_object *soo;
@@ -2272,6 +2300,7 @@ SO_METHOD(setTimestamp)
 
 	RETURN_TRUE;
 }
+/* }}} */
 
 /* {{{ proto bool OAuth::setToken(string token, string token_secret)
    Set a request or access token and token secret to be used in subsequent requests */
@@ -2487,6 +2516,8 @@ SO_METHOD(getLastResponse)
 }
 /* }}} */
 
+/* {{{ proto string getLastResponseHeaders(void)
+Get headers for last response */
 SO_METHOD(getLastResponseHeaders)
 {
 	php_so_object *soo;
@@ -2501,6 +2532,7 @@ SO_METHOD(getLastResponseHeaders)
 	}
 	RETURN_FALSE;
 }
+/* }}} */
 
 /* {{{ proto string OAuth::getRequestHeader(string http_method, string url [, string|array extra_parameters ])
    Generate OAuth header string signature based on the final HTTP method, URL and a string/array of parameters */
@@ -2661,38 +2693,58 @@ static zend_function_entry so_functions[] = { /* {{{ */
 /* }}} */
 
 
+#if PHP_VERSION_ID < 80000
 zval *oauth_read_member(zval *obj, zval *mem, int type, void **cache_slot, zval *rv) /* {{{ */
 {
+	php_so_object *soo = Z_SOO_P(obj);
+    char *name = Z_STRVAL_P(mem);
+#else
+zval *oauth_read_member(zend_object *obj, zend_string *mem, int type, void **cache_slot, zval *rv) /* {{{ */
+{
+	php_so_object *soo = so_object_from_obj(obj);
+    char *name = ZSTR_VAL(mem);
+#endif
 	zval *return_value = NULL;
-	php_so_object *soo;
-
-	soo = Z_SOO_P(obj);
 
 	return_value = std_object_handlers.read_property(obj, mem, type, cache_slot, rv);
 
-	if(!strcasecmp(Z_STRVAL_P(mem),"debug")) {
+	if(!strcasecmp(name, "debug")) {
 		convert_to_boolean(return_value);
 		ZVAL_BOOL(return_value, soo->debug);
-	} else if(!strcasecmp(Z_STRVAL_P(mem),"sslChecks")) {
+	} else if(!strcasecmp(name, "sslChecks")) {
 		ZVAL_LONG(return_value, soo->sslcheck);
 	}
 	return return_value;
 } /* }}} */
 
-static void oauth_write_member(zval *obj, zval *mem, zval *value, void **cache_slot) /* {{{ */
-{
-	char *property;
-	php_so_object *soo;
 
-	property = Z_STRVAL_P(mem);
-	soo = Z_SOO_P(obj);
+#if PHP_VERSION_ID < 80000
+static
+#if PHP_VERSION_ID >= 70400
+zval *
+#else
+void
+#endif
+oauth_write_member(zval *obj, zval *mem, zval *value, void **cache_slot) /* {{{ */
+{
+	php_so_object *soo = Z_SOO_P(obj);
+	char *property = Z_STRVAL_P(mem);
+#else
+static zval *oauth_write_member(zend_object *obj, zend_string *mem, zval *value, void **cache_slot) /* {{{ */
+{
+	php_so_object *soo = so_object_from_obj(obj);
+	char *property = ZSTR_VAL(mem);
+#endif
 
 	if(!strcmp(property,"debug")) {
 		soo->debug = Z_TYPE_P(value) == IS_TRUE ? 1 : 0;
 	} else if(!strcmp(property,"sslChecks")) {
 		soo->sslcheck = Z_LVAL_P(value);
 	}
-	std_object_handlers.write_property(obj, mem, value, cache_slot);
+#if PHP_VERSION_ID >= 70400
+	return
+#endif
+			std_object_handlers.write_property(obj, mem, value, cache_slot);
 } /* }}} */
 
 /* {{{ PHP_MINIT_FUNCTION
@@ -2801,7 +2853,6 @@ PHP_MINFO_FUNCTION(oauth)
 #else
 	php_info_print_table_row(2, "Request engine support", "php_streams");
 #endif
-	php_info_print_table_row(2, "source version", "$Id$");
 	php_info_print_table_row(2, "version", OAUTH_EXT_VER);
 	php_info_print_table_end();
 }
